#[cfg(feature = "audio")]
pub mod audio;
#[cfg(feature = "batches")]
pub mod batches;
#[cfg(feature = "beta")]
pub mod beta;
#[cfg(feature = "chat")]
pub mod chat;
#[cfg(feature = "embeddings")]
pub mod embeddings;
#[cfg(feature = "files")]
pub mod files;
#[cfg(feature = "fine-tuning")]
pub mod fine_tuning;
#[cfg(feature = "images")]
pub mod images;
#[cfg(feature = "models")]
pub mod models;
#[cfg(feature = "moderations")]
pub mod moderations;
#[cfg(feature = "responses")]
pub mod responses;
#[cfg(feature = "uploads")]
pub mod uploads;
