pub mod audio;
pub mod batch;
pub mod beta;
pub mod chat;
pub mod common;
pub mod embedding;
pub mod file;
pub mod fine_tuning;
pub mod image;
pub mod model;
pub mod moderation;
pub mod realtime;
pub mod responses;
pub mod upload;
